@@ -21,7 +21,7 @@ bootstrap IPC::ChildSafe;
 var_ChildSafe_init();
 
 # The current version and a way to access it.
-$VERSION = "2.29"; sub version {$VERSION}
+$VERSION = "2.30"; sub version {$VERSION}
 
 ########################################################################
 # Just a thin layer over child_open (see child.c). Optional last
@@ -321,12 +321,125 @@ really a count of the error messages produced by it.  The programmer
 can optionally register his/her own "discriminator function" for
 determining which output to stderr constitutes an error message.
 
+    This implementation solves the problem, at least for a subset of
+possible child programs, by using a little trick:  it sends a 2nd
+(trivial) command down the pipe right in back of every real command.
+When we see the the output of this special command in the return pipe,
+we know the real command is done.
+
+=head1 CONSTRUCTOR
+
+The constructor takes 3 arguments plus an optional 4th: the 1st is the
+program to run, the 2nd is a command to that program which produces a
+unique one-line output, the 3rd is that unique output, and the 4th is a
+code ref for a discriminator function. If no discriminator is supplied
+then a standard internal one is used.
+
+The 2nd arg is called the "tag command"; preferably this would be
+something lightweight, e.g. a shell builtin. Unfortunately the current
+version has no support for a multi-line return value since this would
+require some fairly complex buffering.
+
+The discriminator function is invoked after each command completes, and
+is passed a reference to an array containing the stderr generated by
+that command in its first parameter. A pointer to the stdout is
+similarly supplied in the second param. Normally this function would
+just apply a regular expression to one or both of these and indicate by
+its return status whether it considers this to constitute an error
+condition. E.g. the version provided internally is:
+
+    sub errors {
+	my($r_stderr, $r_stdout) = @_;
+	grep(!/^\+\s|warning:/i, @$r_stderr);
+    }
+
+    my $sh = IPC::ChildSafe->new('sh', 'echo ++EOT++', '++EOT++', \&errors);
+
+which treats ANY output to stderr as indicative of an error, with the
+exception of lines beginning with "+ " (shell verbosity) or containing
+the string "warning:".
+
+=head1 METHODS
+
+=over 4
+
+=item * B<cmd>
+
+Send specified command to child process. Return behavior varies
+with context:
+
+=over 4
+
+=item array context
+
+returns a hash containing an array of stdout results (key: 'stdout'),
+an array of stderr messages ('stderr), and the "return code" of the
+command ('status').
+
+=item scalar context
+
+returns command's "return code", stores stdout in object for later
+access via method 'stdout'. By default, sends stderr results directly
+to parent's stderr but if '=> STORE' is passed, stderr is stored just
+like stdout. With '=> IGNORE', throws away both stdout and stderr.
+
+=item void context
+
+similar to scalar mode but exits on nonzero return code unless '=>
+IGNORE' is passed.
+
+=item void context and no args
+
+throw away all stored results.
+
+=back
+
+=item * B<stdout>
+
+Return stored output from previous command(s). Return behavior
+varies with context:
+
+=over 4
+
+=item array context
+
+shifts all stored lines off the stdout stack and returns them in a list.
+
+=item scalar context
+
+returns the number of lines currently stored in the stdout stack.
+
+=item void context
+
+prints the current stdout stack to actual stdout.
+
+=back
+
+=item * B<stderr>
+
+Similar to C<stdout> method above. But note that by default stderr does
+not go to the accumulator, but rather to the parent's stderr.  Set the
+STORE attribute to leave stderr in the accumulator instead where this
+method can operate on it.
+
+=item * B<status>
+
+Pass the current stdout and stderr arrays to the currently-registered
+error discriminator and return its results (aka the error count).
+
+=item * B<finish>
+
+Ends the child process and returns its exit status.
+
+=back
+
 =head1 AUTHOR
 
 David Boyce dsb@world.std.com
 
 =head1 SEE ALSO
 
-perl(1), "perldoc IPC::Open3"
+perl(1), "perldoc IPC::Open3", _Advanced Programming in the Unix Environment_
+by W. R. Stevens
 
 =cut
